package BBS;
our $NOLOAD = 1;
#===========================================================
# BBS.pm コアシステム
#
our $VERSION = "0001.20200226.1716";		# 新版
#===========================================================
use utf8;
use strict;
use warnings;
use IO::Select;
use IO::Socket;
use Scalar::Util;
use Encode qw( encode decode );
use Data::Dumper;
use Debug;

use BBS::Handler;						# ハンドラ制御
use BBS::Node;							# ノード制御
use BBS::Timer;							# タイマ
use BBS::IO::Input;						# 入力制御
use BBS::IO::Output;					# 出力制御


#==================================================
# ■ コンストラクタ
# new()
#==================================================
sub new {
	my $class = shift;
	my $self = { @_ };
	bless($self, $class);
	$self->_init();
	return $self;
}

#==================================================
# ■ 属性の初期化
# _init()
#==================================================
sub _init {
	my $self = shift;

	#-------------------------------
	# ● システム変数
	#-------------------------------
		$self->{'select'}							= undef;			# セレクタ
		$self->{'from'}								= undef;			# イベントを発生したノード番号
		$self->{'sigterm'}							= 0;				# サーバ停止フラグ
		$self->{'lockfile'}							= ".lock";			# ロックファイル

	#-------------------------------
	# ● システムハンドラ(handler)
	#-------------------------------

		# サーバイベントハンドラ
		#---------------------------------
		$self->{'handler'}->{'onConnect'}			= {};				# onConnectハンドラ
		#						->{handler_name}	= sub { ... };
		$self->{'handler'}->{'onAccess'}			= {};				# onAccessハンドラ【※廃止方向※】
		#						->{handler_name}	= sub { ... };
		$self->{'handler'}->{'onDisconnect'}		= {};				# onDisconnectハンドラ
		#						->{handler_name}	= sub { ... };

		# 通信関連ハンドラ
		#---------------------------------
		$self->{'handler'}->{'onSend'}				= undef;			# onSendハンドラ
		#											= sub { ... };
		$self->{'handler'}->{'onRecv'}				= undef;			# onRecvハンドラ
		#											= sub { ... };

		# サーバ処理(Serve)関連ハンドラ
		#---------------------------------
		$self->{'handler'}->{'Input'}				= undef;			# Inputハンドラ(入力処理)
		#											= sub { ... };
		$self->{'handler'}->{'Output'}				= undef;			# Outputハンドラ(出力処理)
		#											= sub { ... };
		$self->{'handler'}->{'SysWork'}				= {};				# SysWorkハンドラ（システム処理）
		#						->{handler_id}		= sub { ... };

	#-------------------------------
	# ● ノードテーブル(node)
	#-------------------------------
		$self->{'__Node'}							= new BBS::Node;			# ノード制御

	# ※以下のパラメータ名はシステム専用です。
	# 同じパラメータ名は使用しないでください。
	# 				->node(n)
	#					->{'__Socket'}				= $sock;					# ソケットオブジェクト
	#					->{'AppWork'}				= new BBS::Handler();		# AppWorkハンドラ（アプリケーション処理）
	#					->{'Disconnected'}			= n;						# 切断イベントフラグ
}

#==================================================
# 【 ノード操作 】
#==================================================
# ■ 接続ノードリストを得る
# nodes()
#
# [引数]
# なし
#
# [返り値]
# r = [ num1, num2, ... ]		ノード番号(接続順)
#
# 接続しているノードリストを接続順の配列で取得します。
#==================================================
sub nodes {
	printf "\n(%s::nodes [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;

	my @r = $self->{'__Node'}->nodes();											# 接続順リストから取得
	printf "\n** nodes=[ %s ] ", join(',', @r) if ($::DEBUG =~ /b/);			# ???? 接続リスト
	return @r;
}

#==================================================
# ■ ノードテーブルのデータ領域を返す
# node( num )
#
# [引数]
# num : ノード番号
#
# [返り値]
# r =
#       undef : 失敗
# {データ領域} : 成功
#
# ノードテーブルのデータ領域を返します。
#
# ノードテーブルはそれぞれのノードが保持するデータを保持する領域で、
# 新たなノードが接続するとノードのためのデータ領域がノードテーブルに用意されます。
# 接続中のノードが切断を試みるとノードが保持するデータ領域がノードテーブルから削除されます。
#
# ノードテーブルはハッシュ変数ですので、返り値に対して操作することで、
# データを追加、削除することができます。
#
# ノードテーブルはシステムが使用するデータも保存されるため、
# 誤ってシステムデータを削除したり書き換えたりしないよう、十分注意する必要があります。
# システムが使用するデータは、init()のノードテーブルに記されています。
#==================================================
sub node {
	printf "\n(%s::node [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	my $num = shift;		# ノード番号

	return $self->{'__Node'}->node($num);
}

#==================================================
# 【 ハンドラ操作 】
#==================================================
# ■ アプリケーションハンドラ設定
# sethandler( func, [num] )
#
# ※関数名変更により設置
#==================================================
sub sethandler {
	printf "\n(%s::sethandler [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);	# ????
	my $self = shift;
	my $func = shift;		# コールバック関数
	my $num = shift;		# ノード番号
		$num = $self->{'from'} unless defined($num);				# ノード番号が未定義なら応対中のノード番号を充てる

	return $self->setapphandler( $func, $num );
}

#==================================================
# ■ アプリケーションハンドラ設定
# setapphandler( func, [num] )
#
# [引数]
# func : サブルーチン ( func = sub { ... } )
#  num : ノード番号
#
# [返り値]
# r =
# undef : 失敗
#     1 : 成功
#
# アプリケーションハンドラは、アプリケーション処理を行うためのハンドラで、
# 各ノード、１つずつ所有します。
# このハンドラでは一般的に、ユーザの要求に応答するアプリケーション処理が設定されます。
# 設定されたルーチンはサーバ処理(serve)のアプリケーション処理の段階で呼び出しが行われます。
#==================================================
sub setapphandler {
	printf "\n(%s::sethandler [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);	# ????
	my $self = shift;
	my $func = shift;
	my $num = shift;
		$num = $self->{'from'} unless defined($num);				# ノード番号が未定義なら応対中のノード番号を充てる

	return unless ( ref($func) eq 'CODE' );							# ルーチンが未定義(undef)
	return unless ( defined( $self->node($num) ) );					# ノードが開いていない(undef)

	printf "\n** num=(%d) ", $num if ($::DEBUG =~ /b/);				# ???? ノード番号
	$self->node($num)->{'AppWork'}->set($func);						# ハンドラにルーチンを設定
	return 1;
}

#==================================================
# ■ システムハンドラ設定
# setsyshandler( handler_name, func )
#
# [引数]
# handler_name : ハンドラ名
#         func : ハンドラルーチン ( func = sub { ... } )
#
# [返り値]
# r =
# undef : 失敗
#     1 : 成功
#
# システムハンドラは、サーバ（システム）側が持ちます。
#
# onSend, onRecv, Input, Output の各ハンドラでは、設定できるルーチンが
# それぞれ１つしか設定できませんので、ルーチンが既に設定されているところで、
# 新たなルーチンを設定すると前のルーチンは解除されます。
#
# onConnect, onAccess, onDisconnect, SysWork の各ハンドラは、
# 一つのハンドラに複数のルーチンを設定できますが、ルーチンの呼び出し順序はランダムで決定しているので、
# 呼び出し順序に影響するような処理を行うルーチンを定義すると誤動作を起こす原因になります。
# また、設定するルーチンは、同じルーチンを何度も設定しても連続して登録されないため、
# ハンドラが呼び出されても、呼び出されるルーチンは１回のみとなります。
#
# ハンドラに設定したルーチンは削除することはできません。
# １つのルーチンしか設定できないハンドラでも、上書き設定は可能ですが、ハンドラから削除することはできません。
#
# ※ハンドラの仕組みについては、解説サイトをご覧ください。
#==================================================
sub setsyshandler {
	printf "\n(%s::setsyshandler [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	my $name = shift;		# ハンドラ名
	my $func = shift;		# ハンドラルーチン

	   if ( $name =~ /onconnect/i )    { $name = 'onConnect';    }
	elsif ( $name =~ /onaccess/i )     { $name = 'onAccess';     }
	elsif ( $name =~ /ondisconnect/i ) { $name = 'onDisconnect'; }
	elsif ( $name =~ /onsend/i )       { $name = 'onSend';       }
	elsif ( $name =~ /onrecv/i )       { $name = 'onRecv';       }
	elsif ( $name =~ /input/i )        { $name = 'Input';        }
	elsif ( $name =~ /output/i )       { $name = 'Output';       }
	elsif ( $name =~ /syswork/i )      { $name = 'SysWork';      }
	else                               { $name = undef;          }

	return if ( (defined($name) == 0) || ($name eq '') );									# ハンドラ名が未定義またはヌル(undef)
	return unless ( ref($func) eq 'CODE' );													# ルーチンが未定義(undef)

	my $hid = Scalar::Util::refaddr( $func );												# ルーチンのアドレス(=ID)を取得
	printf "\n** name=[%s] ", defined($name) ? $name : 'undef' if ($::DEBUG =~ /b/);		# ???? ハンドラ名
	printf "handler_id=[%s]", defined($hid) ? $hid : 'undef' if ($::DEBUG =~ /b/);			# ???? ルーチンアドレス

	# Input, Output, onRecv, onSend
	if ($name =~ /(Input|Output|onRecv|onSend)/ ) {											# ルーチンを１つだけ設定できるハンドラ
		$self->{'handler'}->{$name} = new BBS::Handler($func);								# ルーチンを新たに設定
	}
	# onConnect, onAccess, onDisconnect, SysWork
	else {																					# 多数のルーチンを設定できるハンドラ
		$self->{'handler'}->{$name}->{$hid} = new BBS::Handler($func);						# ルーチンを新たに設定
	}
}

#==================================================
# ■ モジュールに定義されているローダルーチンを呼び出す
# _load()
#
# [引数]
# なし
#
# [返り値]
# なし
#
# ※この関数は、start()から呼び出されるためもので、個別に呼び出す必要はありません。
#
# ロードしたモジュール(パッケージ内)に
# ローダルーチン( load() )が定義されているものを全て呼び出します。
#
# ローダルーチンはモジュール(パッケージ)の初期設定を行うための専用ルーチンで、
# サーバの開始直前で呼び出しを行いますが、
# モジュール(パッケージ内)に $NOLOAD=1 があると、ローダルーチンの呼び出しを行いません。
#
# ●モジュール作成の注意点
#
# ローダルーチンはパッケージ名とファイル名(パス)を元に呼び出しを行うため、
# モジュールを配置するファイル名(パス)とパッケージ名をそろえておいてください。
#==================================================
sub _load {
	printf "\n(%s::_load [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);			# ????
	my $self = shift;

	my ($pkg) = caller;
	if ($pkg ne 'BBS') {
		print encode( 'cp932', "(error): _load()はBBSモジュール以外から呼び出すことはできません.\n");
		exit(0);
	}

	# print map { "\n==> p=[$_] " } @INC if ($::DEBUG =~ /c/);							# ???? インクルードパス
	# print "\n" if ($::DEBUG =~ /c/);													#

	# my $k (sort(keys(%INC))) {														# ???? ロードしたパッケージ ($k:パッケージのパス, $v:ファイルが存在するパス)
	# 	printf "\n==> k=[%s], v=[%s] ", $k, $INC{$k} if ($::DEBUG =~ /c/);				#
	# }																					#
	# print "\n" if ($::DEBUG =~ /c/);													#

	foreach my $pkgname ( sort(keys(%INC)) ) {											# ローダの対象をパッケージ名でフィルタリング
		next if grep { $INC{$pkgname} =~ /^$_/ } @INC;									# パッケージパスがシステムモジュールなら除外
		$pkgname =~ s/\.pm//;															# 拡張子を除去
		$pkgname =~ s/\//::/g;															# パス文字を除去
		my $pkg = $pkgname;
		# printf "\n==> p=[%s] ", $pkg if ($::DEBUG =~ /c/);							# ???? ローダの対象パッケージ

		no strict 'refs';																# 怒られるので遮断
		my $version = ${ $pkg."::VERSION" };											# パッケージのバージョン番号
		# unless ( defined($version) ) {
		# 	$version = '';																#
		# 	print "\n[undef] " if ($::DEBUG =~ /c/);									# ???? 未定義
		# }
		# printf "\n** ver=(%s) ", $version if ($::DEBUG =~ /c/);						# ????

		my $noload = ${ $pkg."::NOLOAD" };												# ローダ抑止フラグ
		# unless ( defined($noload) ) {
		# 	$noload = 0;																#
		# 	printf "\n[undef] " if ($::DEBUG =~ /c/);									# ???? 未定義
		# }
		# printf "\n** noload=(%s) ", $noload if ($::DEBUG =~ /c/);						# ????

		unless ( $noload ) {															# ローダ抑止フラグが１でなければ
			my $load = $pkg.'::load($self)';											#
			my $r = eval("$load");														# モジュールにload()があれば呼び出す
			# printf "\n*** load.r=(%d) ", $r if defined($r) && ($::DEBUG =~ /b/);		# ???? ローダの結果
		}
	}
	return;
}

#==================================================
# ■ イベントが発生したときに応対するノード番号を返す
# num = from()
#
# [引数]
# なし
#
# [返り値]
# num = ノード番号
#==================================================
sub from {
	# printf "\n(%s::from [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	#printf "\nfrom=(%d) ", $self->{'from'} if ($::DEBUG =~ /b/);								# ????
	return $self->{'from'};
}

#==================================================
# ■ CTRL+Cによるプログラム停止を無効にする
# termtrap()
#
# [引数]
# なし
#
# [返り値]
# なし
#
# CTRL+Cによるプログラム停止を抑止します。
# 一度呼び出すと機能を取り消すことができません。
# プログラムを停止する場合は、実行するメインプログラムのパスに
# 作成されるロックファイルを削除します。
#==================================================
sub termtrap {
	printf "\n(%s::termtrap [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	$SIG{'ALRM'} = sub { return };
	$SIG{'INT'}  = sub { return };
	$SIG{'HUP'}  = sub { return };
	$SIG{'QUIT'} = sub { return };
	$SIG{'TERM'} = sub { return };
	return;
}

#==================================================
# ■ サーバの開始
# start( port )
#
# [引数]
# port : ポート番号
#
# [返り値]
# なし
#
# 指定したポート番号でサーバを開始し、ノードからの着信を待ちます。
# ノードからの着信、通信によってイベントハンドラを呼び出します。
#==================================================
sub start {
	printf "\n(%s::start [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	my $port = shift;
	if ( (defined($port) != 1 ) || ($port !~ /^\d+$/) ) {			# ポート番号が無効
		print encode( 'cp932', "(error): ポート番号を指定してください.\n" );
		exit(0);
	};

	# (1) ロックファイル作成
	open(my $fh, ">$self->{'lockfile'}");
	close($fh);

	# (2) 各モジュールのローダを実行
	$self->_load();

	# (3) サーバ処理開始
	my $listen = new IO::Socket::INET(Listen => 1, LocalPort => $port);				# ソケットリスナー(3)
	$self->{'select'} = new IO::Select($listen);									# セレクタ

	$| = 1;
	while ( $self->{'sigterm'} == 0 ) {												# 終了シグナルが発生したらループを抜ける
		my @ready = $self->{'select'}->can_read(0.0001);							# 通信イベントを監視
		printf "\n** ready=(%d) ", $#ready+1 if ($::DEBUG =~ /b/);					# ???? 行列待ちに入っているソケット数

		if ( $#ready >= 0 ) {														# 通信発生
			foreach my $socket (@ready) {											# 行列待ちを捌く
				if ($socket == $listen) {											# 【 新たなクライアントが接続した(onConnect) 】
					#==============================================
					# ■■ ソケット接続イベント処理(onConnect) ■■
					#==============================================
					print "\n\n-----[ onConnect ]----- " if ($::DEBUG =~ /b/);		# ????
					$socket = $listen->accept;										# リスナーをacceptして新しいソケットを作り
					$self->{'select'}->add($socket);								# そのソケットをセレクタに追加
					my $nodeno = $socket->fileno;									# ソケットのファイル番号を得る（ファイル番号＝ノード番号）
					$self->{'from'} = $nodeno;										# 応対中ノードの番号をfrom()にセット
					printf "\n** nodeno=(%d)\n", $nodeno if ($::DEBUG =~ /b/);		# ???? ノード番号

					my $n = $self->{'__Node'};
					$n->add($nodeno);												# ノードを追加
					$n->node($nodeno)->{'__Socket'} = $socket;						# ソケットオブジェクトをノードに保存
					$n->node($nodeno)->{'AppWork'} = new BBS::Handler();			# アプリケーションハンドラをノードに保存
					$self->_onconnect();											# onConnectイベント処理へ
				}
				else {																# 【 接続するクライアントから通信が発生した(onAccess) 】
					#===========================================
					# ■■ データ通信イベント処理(onAccess) ■■
					#===========================================
					print "\n\n-----[ onAccess ]----- " if ($::DEBUG =~ /b/);		# ????
					my $nodeno = $socket->fileno;									# ソケットのファイル番号を得る（ファイル番号＝ノード番号）
					$self->{'from'} = $nodeno;										# 応対中ノードの番号をfrom()にセット
					printf "\n** nodeno=(%d)\n", $nodeno if ($::DEBUG =~ /b/);		# ???? ノード番号

					my $rcvdata;													# 受信バッファ
					$socket->recv($rcvdata, 4096);									# 受信データを取得

					if ($rcvdata eq '') {											# 受信バッファが空(通信エラー)
						$self->_ondisconnect( $nodeno );							# onDisconnectイベント処理へ
					}
					else {															# 受信バッファにデータがある
						$self->_onrecv( $rcvdata );									# onrecvイベント処理へ
						#$self->_onaccess();										# onaccessイベント処理へ【廃止】
					}
				}
				$self->_serve( $self->{'from'} );									# サーバ処理へ
			}
		}
		else {																		# 【 サーバが待機中 】
			#===================================
			# ■■ サーバ処理処理(Serve) ■■
			#===================================
			$self->_serve();														# サーバ処理へ
		}
	}																				# 停止フラグが立たなければサーバループに戻る
	return 0;																		# 停止フラグが立たてばサーバを停止
}

#==================================================
# ■ サーバ処理
# _serve( nodeno )
#
# [引数]
# nodeno : 処理を開始するノード番号(省略可)
#
# [返り値]
# なし
#
# ※この関数は、start()から呼び出されるもので、通常は呼び出し禁止です。
#
# サーバ処理に関わるハンドラを呼び出します。
# 
# この処理では、ノードから受け取ったデータをアプリケーションに渡す入力処理(Input)、
# サーバからノードにデータを送出する出力処理(Output)、
# アプリケーションのシステム処理(SysWork), アプリケーション処理(AppWork)の
# ４つのハンドラが呼び出されます。
#
# ハンドラの呼び出しはノードが接続した順で行われます。
# 
# 例えば、接続しているノード接続順が 4, 5, 7, 6 として、
# 引数を指定すると、指定するノード番号から処理を開始し、一巡するように呼び出しを行います。
# 引数に7が指定されているなら、7→6→4→5の順番で呼び出しが行われます。
# 引数を省略した場合は、ノードの接続順のとおり、4→5→7→6の順番で呼び出しが行われます。
#
# 詳細説明はサーバ処理の解説ページをご覧ください。
#==================================================
sub _serve {
	printf "\n(%s::_serve [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	print "\n\n-----[ serve ]----- " if ($::DEBUG =~ /b/);											# ????
	my $self = shift;
	my $nodeno = shift;				# 呼び出し開始ノード
	my @served;						# 実行済みリスト( $served[nodeno] = flag )
	my $exit = 0;					# 終了フラグ
	my $running = 0;				# 実行中ノード番号(デフォルト:0)

	# ロックファイルが消失したら終了シグナルを発生、プログラムを停止
	$self->{'sigterm'} = 1 unless ( -e "$self->{'lockfile'}" );

   	my @nodes = $self->{'__Node'}->nodes();			# 接続順リスト取得
	return if ( $#nodes < 0 );						# ノードが一つも接続されていない(undef)


	# ノードが指定されていたら、指定ノードの順番（順位）を取得
	# [ 4, 5, 7, 6 ] ; 4 = [0] ; 5 = [1] ; 7 = [3] ; 6 = [4]
	if ( defined($nodeno) ) {
		my ($order) = grep { $nodes[$_] eq $nodeno } 0..$#nodes;
		$running = $order if defined($order);							# 順位が得られたら、実行中ノード番号を更新
	}

	# ノードのハンドラを順番に呼び出す
	until ( $exit ) {																										# 終了フラグが立ったら終了
		printf "\n\n** node=(%d), order=[%d], exit=(%d) ", $nodes[$running], $running, $exit if ($::DEBUG =~ /b/);			# ???? 実行中のノード番号, ノードの実行順位, 終了フラグ
		$self->{'from'} = $nodes[$running];																					# 実行中のノード番号をfrom()にセット

		#==========================
		# ■■ (1) 入力処理(Input) ■■
		#==========================
		print "\n\t{{ Input }} " if ($::DEBUG =~ /b/);														# ????
		#print "\n\t** ".Dumper( $self->{'handler'}->{'Input'} ) if ($::DEBUG =~ /c/);						# ???? Inputハンドラの内容

		if ( defined($self->{'handler'}->{'Input'}) ) {
			my $data = $self->{'handler'}->{'Input'}->call();												# Inputハンドラを呼び出す
		}

		#===========================
		# ■■ (2) 出力処理(Output) ■■
		#===========================
		print "\n\t{{ Output }} " if ($::DEBUG =~ /b/);														# ????
		#print "\n\t** ".Dumper( $self->{'handler'}->{'Output'} ) if ($::DEBUG =~ /c/);						# ????

		if ( defined($self->{'handler'}->{'Output'}) ) {
			my $data = $self->{'handler'}->{'Output'}->call();						 						# Outputハンドラを呼び出す
			if ( defined($data) && ($data ne '') ) {																			# 返り値にデータが含まれている
				#printf "\n** data.length=(%d) ", length($data) if defined($data) && ($::DEBUG =~ /b/);			# ???? 送信データ長
				$self->node( $self->{'from'} )->{'__Socket'}->send($data);										# データを送出
			}
			else {																							# データが含まれてていない	
				print "\n ** data nothing ---- " if ($::DEBUG =~ /b/);											# ????
			}
		}

		#===============================
		# ■■ (3) システム処理(SysWork) ■■
		#===============================
		print "\n\t{{ SysWork }} " if ($::DEBUG =~ /b/);													# ????
		print "\n\t** ".Dumper( $self->{'handler'}->{'SysWork'} ) if ($::DEBUG =~ /c/);						# ????

		foreach my $hid ( keys(%{ $self->{'handler'}->{'SysWork'} }) ) {
			printf "\n** hid=(%s) ", $hid if ($::DEBUG =~ /b/);												# ???? ハンドラID
			if ( exists($self->{'handler'}->{'SysWork'}->{$hid}) ) {										# ハンドラが定義されている
				print " calling" if ($::DEBUG =~ /b/);															# ????
				$self->{'handler'}->{'SysWork'}->{$hid}->call();												# SysWorkハンドラを呼び出す
			}
			else {																							# ハンドラが未定義
				print " no calling" if ($::DEBUG =~ /b/);														# ????
			}
		}

		#======================================
		# ■■ (4) アプリケーション処理(AppWork) ■■
		#======================================
		print "\n\t{{ AppWork }} " if ($::DEBUG =~ /b/);													# ????
		print "\n\t** ".Dumper( $self->node( $self->{'from'} )->{'AppWork'} ) if ($::DEBUG =~ /c/);			# ????

		if ( defined( $self->node( $self->{'from'} )->{'AppWork'} ) ) {
			$self->node( $self->{'from'} )->{'AppWork'}->call();											# AppWorkハンドラを呼び出す
		}

		$served[$running] = 1;																				# 実行済みリストにノード番号を追加

		# 巡回終了チェック
		$running++;																							# ノードの実行順位＋１
		$running = ( $running > $#nodes ) ? 0 : $running;													# ノードの実行順位がノード総数を超えたら０をセット
		$exit = 1 if ( defined($served[$running]) );														# 実行するノードが実行済みリストに追加されているなら終了
		print "\n" if ($::DEBUG =~ /b/);																	# ????
	}																										# 【 ループの終点 】
	printf "\n** exit serve " if ($::DEBUG =~ /b/);															# ???? サーバ処理終了
}

#==================================================
# ■ 接続イベント処理
# _onconnect()
#
# [引数]
# なし
#
# [返り値]
# なし(undef)
#
# ※この関数は、start()から呼び出されるためのもので、直接呼び出し禁止です。
#
# この関数は新たなノードがサーバに接続されたときに呼び出されます。
#==================================================
sub _onconnect {
	printf "\n(%s::_onconnect [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	printf "\n** node=(%d) ", $self->{'from'} if ($::DEBUG =~ /b/);										# ???? イベントを発動したノード番号
	print "\n\t**".Dumper( $self->{'handler'}->{'onConnect'} ) if ($::DEBUG =~ /b/);					# ???? ハンドラの内容

	foreach my $hid ( keys(%{ $self->{'handler'}->{'onConnect'} }) ) {
		printf "\n** hid=(%s) ", $hid if ($::DEBUG =~ /b/);												# ???? ハンドラID
		if ( defined($self->{'handler'}->{'onConnect'}->{$hid}) ) {
			print " calling" if ($::DEBUG =~ /b/);														# ???? 呼び出す
			$self->{'handler'}->{'onConnect'}->{$hid}->call();											# onConnectハンドラを呼び出す
		}
		else {								
			print " no calling" if ($::DEBUG =~ /b/);													# ???? 呼び出せなかった
		}
	}
}

#==================================================
# ■ 通信イベント処理	【 ※廃止※ 】
# _onaccess()
#
# [引数]
# なし
#
# [返り値]
# なし
# 
# データ受信はonRecvに変わり、ここは不要になったので廃止しました。
#==================================================
# sub _onaccess {
# 	printf "\n(%s::_onaccess [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);			# ????
# 	my $self = shift;
# 	printf "\n*** node=(%d) ", $self->{'from'} if ($::DEBUG =~ /b/);									# ???? イベントを発動したノード番号
#	print "\n\t**".Dumper( $self->{'handler'}->{'onAccess'} ) if ($::DEBUG =~ /b/);						# ???? ハンドラの内容

# 	if ( defined($self->{'handler'}->{'onAccess'}) ) {
# 		$self->{'handler'}->{'onAccess'}->call();							 							# onAccessハンドラを呼び出す
# 	}
# }

#==================================================
# ■ 受信イベント(onRecv)処理
# _onrecv( data )
#
# [引数]
# data : 受信データ
#
# [返り値]
# なし
#
# ※この関数は、start()から呼び出されるためのもので、直接呼び出し禁止です。
#
# この関数はonAccessイベントが発生、データを取得したときに呼び出されます。
# 取得した受信データは受信ハンドラ(onRecv)に渡されます。
#==================================================
sub _onrecv {
	printf "\n(%s::_onrecv [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);			# ????
	my $self = shift;
	my $rcvdata = shift;				# 受信データ

	printf "\n*** node=(%d) ",              $self->{'from'} if ($::DEBUG =~ /b/);						# ???? イベントを発動したノード番号
	printf "\n*** buf_recv.length=(%d) ",   length( $rcvdata ) if ($::DEBUG =~ /b/);					# ???? 受信バッファのデータ長
	printf "\n*** buf_recv=[%s] ",          $rcvdata if ($::DEBUG =~ /b/);								# ???? 受信データの内容

	if ( defined($self->{'handler'}->{'onRecv'}) ) {
		$self->{'handler'}->{'onRecv'}->call( $rcvdata );												# onRecvハンドラを呼び出す
	}
}

#==================================================
# ■ 切断イベント処理
# _ondisconnect( num )
#
# [引数]
# num : ノード番号
#
# [返り値]
# r =
#  1 : 正常終了
# -1 : 引数が指定されていない
# -2 : ノードが開かれていない
# -3 : すでに切断イベントを通過している
#
# ※この関数は、start()から呼び出されるためのもので、直接呼び出し禁止です。
# ハンドラなどからノードの切断を行う場合は disconnect() を使用します。
#
# 切断イベントハンドラ(onDisconnect)を呼び出し、
# ハンドラ処理が戻るとソケットが廃棄され、ノードの整理が行われます。
#==================================================
sub _ondisconnect {
	printf "\n(%s::_ondisconnect [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	print "\n\n-----[ onDisconnect ]----- " if ($::DEBUG =~ /b/);										# ????
	my $self = shift;
	my $num = shift;					# ノード番号

	return -1 unless defined($num);																		# 引数が指定されていない(-1)
	return -2 unless ( defined( $self->node($num) ));													# ノードが見つからない(-2)

	# onDisconnectハンドラ内からの再呼び出し防止
	my $disconnected = $self->node($num)->{'Disconnected'};												# 切断イベントフラグを参照
	return -3 if defined($disconnected);																# すでに切断イベントを通過している(-3)
	$self->node($num)->{'Disconnected'} = 1;															# 切断イベントフラグを立てる

	# onDisconnectハンドラ呼出し
	$self->{'from'} = $num;																				# 切断するノード番号をfromに設定
	foreach my $hid ( keys(%{ $self->{'handler'}->{'onDisconnect'} }) ) {
		printf "\n** hid=(%s) ", $hid if ($::DEBUG =~ /b/);												# ???? ハンドラID
		if ( defined($self->{'handler'}->{'onDisconnect'}->{$hid}) ) {									# ハンドラが定義されている
			print " calling" if ($::DEBUG =~ /b/);															# ???? 呼び出す
			$self->{'handler'}->{'onDisconnect'}->{$hid}->call();											# onDisconnectハンドラを呼び出す
		}
		else {																							# ハンドラが未定義
			print " no calling" if ($::DEBUG =~ /b/);														# ????
		}
	}

	# ソケットの切断
	my $socket = $self->node($num)->{'__Socket'};
	$socket->close;																						# ソケットを閉じる
	$self->{'select'}->remove($socket);																	# セレクタからソケットを削除
	$self->{'__Node'}->del($num);																		# ノードテーブルを廃棄
}

#==================================================
# ■ 切断処理
# disconnect( num )
#
# [引数]
# num : ノード番号
#
# [返り値]
# undef
#
# ログアウトやタイムアウトなど、アプリケーション処理でノードとの切断を行うときに呼び出します。
#
# この関数では、切断イベント処理(_ondisconnect)を呼び出します。
#==================================================
sub disconnect {
	printf "\n(%s::disconnect [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);	# ????
	my $self = shift;
	my $num = shift;		# ノード番号

	# onDisconnectハンドラ内からの再呼び出し防止
	return if defined( $self->node($num)->{'Disconnected'} );										# すでに切断イベントを通過している(undef)
	$self->_ondisconnect( $num );																	# 切断イベント処理を呼び出す
}

#==================================================
# ■ データ送信
# from_send( send_data, optional_data )
#
# [引数]
#       send_data : 送信データ
#   optional_data : その他データ
#
# [返り値]
# r =
# -3 : ノードが活動していない
#
# [send()での返り値]
#  1 : 完了
# -1 : 送信データがない
# -2 : ノードが活動していない
#
# 本関数は送信関数send()を呼び出します。
# データ送信関数は送信先を指定する必要がありますが、
# 本関数では、自動的にイベントを発動したノードを送信先として呼び出すため、
# 送信先の指定が不要です。
# ※本関数は、send( $data, from() )と同じ処理を行います。
#==================================================
sub from_send {
	printf "\n(%s::from_send [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	my $to = $self->{'from'};			# 応対中のノード番号を取得

	return -3 unless ( defined($to) );						# ノード番号がない(-3)
	return $self->send( $to, @_ );							# データ送信関数(send())を呼び出す
}

#==================================================
# ■ データ送信
# send( nodeno, data, optional_data )
#
# [引数]
#          nodeno : 送信先ノード番号
#            data : 送信データ
#   optional_data : その他データ
#
# [返り値]
# r =
#  1 : 完了
# -1 : 送信データがない
# -2 : ノードが活動していない
#
# 本関数は送信するデータをデータ送信イベントハンドラ(Send)に渡します。
# 本システムは送信データのためのデータ処理は実装されておらず、処理部分はハンドラ側で行います。
# そのため、処理はハンドラに定義する必要があります。
# 送信の仕組みについては、別途ドキュメントをご覧ください。
#==================================================
sub send {
	printf "\n(%s::send [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	my $to = shift;						# 送信先ノード
	my $data = shift;					# 送信データ

	return -1 unless ( defined($to) );												# ノード番号がない(-1)
	return -2 if (( defined($data) == 0 ) || ( $data eq '' ));						# データが未定義またはヌル(-2)

	#printf "\n** to=(%d) ", $to if ($::DEBUG =~ /b/);								# ????
	#printf "\n** data=(%d)[%s] ", length($data), $data if ($::DEBUG =~ /b/);		# ????

	return unless ( defined($self->{'handler'}->{'onSend'}) );						# onSendハンドラが未定義(undef)
	return $self->{'handler'}->{'onSend'}->call( $to, $data, @_ );					# 引数をSendハンドラに渡す
}

# printf("\n(%s) [ %s ] ", $VERSION, __PACKAGE__ ) if ( $::DEBUG =~ /a/ );

#================================================================
=pod

=encoding utf8

=head1 スクリプト名

BBS.pm - BBS.pm コアシステム

=head1 著者

naoit0 <https://www.naoit0.com/projects/bbs_pm/>

=cut
1;