package BBS::IO::Input;
our $NOLOAD = 1;
#===========================================================
# BBS.pm コアシステム
# サブモジュール 【 入力制御 】
# 
our $VERSION = "0001.20210306.1722";		# 最新

# 本モジュールはBBS.pmから受け取った受信データ（バイナリ）を
# フィルタリングして文字列などのデータに成形する。
#
# モジュールで扱うデータはＵＴＦ８のため、
# モジュールに入力する前にＵＴＦ８で変換しておく必要がある。
#===========================================================
use utf8;
use strict;
use warnings;
use Encode qw( encode decode );
use Data::Dumper;
use Debug;

#==================================================
# ■ コンストラクタ
# new()
#==================================================
sub new {
	my $class = shift;
	my $self = { @_ };
	bless($self, $class);
	$self->_init();
	return $self;
}

#==================================================
# ■ 属性の初期化
# _init()
#==================================================
sub _init {
	my $self = shift;

	#【バッファ】

		$self->{'recv'}				= [];							# 受信バッファ
																	# @recv = [ packet, packet, ... ]

		$self->{'input'}			= '';							# 入力バッファ
																		# ラインモードではcp932で管理
																		# ブロックモードと兼用

		$self->{'inkey'}			= '';							# 直前に入力した文字

	#【共通パラメータ】

		# $self->{'pause'}			= 0;							# キー入力一時停止フラグ
		# ※ 入れようよ思ったけど、うまく動作できないと思う。
		# ※ 出力と表示のタイミングを確認


		$self->{'mode'}				= 1;							# 入力モード
																		# 1: ラインモード
																		# 2: ブロックモード

		$self->{'input_size'}		= 80;							# 入力バッファサイズ
																		# ラインモード有効時）
																				# 文字数（ＣＰ９３２）
																				# (0: 無制限)
																		# （ブロックモード有効時）
																				# バイト数

	#【ライン入力モード用パラメータ】

		$self->{'input_len'}		= 0;							# 入力バッファのデータ長（ＣＰ９３２）

		$self->{'inkey_len'}		= 0;							# 直前に入力した文字のデータ長（ＣＰ９３２）

		$self->{'echo'}				= 1;							# エコーバック
																		# 0: 無効
																		# 1: 有効

		$self->{'mask'}				= 0;							# マスク処理（エコーバック有効時）
																		# 0: 無効
																		# 1: 有効（マスク文字で置換）
															
		$self->{'maskchr'}			= '*';							# マスク文字(マスク処理有効時)

		$self->{'autocr'}			= 0;							# 自動確定
																	# 入力バッファがデータで満たされたときに自動的に確定する
																		# 0: 無効
																		# 1: 有効

		$self->{'reject_chr'}		= '[\x00-\x1f]';				# 入力バッファの取り込みを拒否する文字(パターンで指定)
																	# デフォルトでは全ての制御文字を入力バッファに取り込まない。
																	# チャットなどでエスケープシーケンス装飾を行う場合は、<ESC> を許可する。
																	# '[\x00-\x1a][\x1c-\x1f]';

		$self->{'iscr'}				= 0;							# 確定フラグ
																	# CRが入力されるとフラグがセットされる。
																	# フラグがセットされたままkeyin()が呼び出されると
																	# 入力バッファがクリアされ、新たなデータが追加される。 

		# $self->{'ctrl-code'}		= [								# 制御文字置換テーブル（テスト用）
		# 	'NUL', 'SOH', 'STX', 'ETX', 'EOT', 'ENQ', 'ACK', 'BEL',
		# 	'BS',  'HT',  'LF',  'VT',  'FF',  'CR',  'SO',  'SI',
		# 	'DLE', 'DC1', 'DC2', 'DC3', 'DC4', 'NAK', 'SYN', 'ETB',
		# 	'CAN', 'EM',  'SUB', 'ESC', 'FS',  'GS',  'RS',  'US'
		# ];
}

#==================================================
# ■ 受信バッファからデータを取り出し、入力バッファに移す
# r = keyin();
# 
# [引数]
# 	なし
# 
# [返り値]
#    data = エコーバック文字列
# 
# 入力モードによって、入力バッファのルールが変わる。
# 
# バイナリデータを受信する場合はブロック入力モードを使用。
# ただしバイナリ転送プロトコルは別途必要。
#
# 文字データを受信する場合はライン入力モードを使用。
# オプション設定により、大抵の入力手法に対応。
#==================================================
sub keyin {
	printf "\n(%s::keyin [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	my $echo;		# エコーバック文字列

	#====================================================
	# 【 ■■■■■■ ブロックモード ■■■■■■ 】
	#====================================================
	if ( $self->{'mode'} eq 'block' )  {

		# ここでは、受信バッファ{'recv'}から、{'block_datasize'}文字分になるまで取り込む。
		# データを取り出して、入力バッファに取り込ませる。
	}

	#====================================================
	# 【 ■■■■■■ ラインモード ■■■■■■ 】
	#====================================================
	else {																						# 【 ラインモード始まり 】
		# printf "\n\n** (line-mode) _____________";											# ????
		# 受信バッファ{'recv'}から１文字を取り出して入力バッファ{'input'}に移す。
		# 受信バッファ{'recv'}はレコード(パケット)ごとに保管されている。
		# 
		# 入力は、先頭のレコードから取り出されるので、
		# 先頭レコードを一時的に取り出し、これを１バイトごとの配列(@packet)に変換する。
		# 
		# ＵＴＦ８の１文字分のデータを@recv_packetの先頭から取り出す。
		# 取り出し途中で@recv_packetのデータがなくなったら、
		# 受信バッファから次のレコードを取り出して補充する。
		# 取り出し完了後、残った@recv_packetは受信バッファの先頭に戻す。
		# 
		# 取り出した１文字分のデータが制御文字(<BS>, <DEL>, <CR>)の場合は、
		# 入力バッファの操作を行う。
		# それ以外のデータは入力バッファ{'input'}への取り込みを試みる。
		# もし取り込んだときに入力バッファがオーバーフローする場合は、取り込みは行わない。
		# オーバーフローしなければ取り込みを行う。
		# 取り込みを行う際、入力するデータをＣＰ９３２での文字数を算出。
		# これは、コンソールに出力する文字数が半角１バイト、全角２バイトで構成していて、
		# 同様の文字コードセットとして、ＣＰ９３２が適しているため。
		# 全角でのデータ長が２～４バイトのように可変すると文字数関わる計算ができないので、
		# 文字列データの管理はＵＴＦ８で行い、文字数はＣＰ９３２を基準に算出する。

		# 確定中なら入力バッファをクリア
		#printf "\n** iscr=(%d) ", $self->{'iscr'};						# ???? 確定フラグ
		if ( $self->{'iscr'} == 1 ) {									# 確定フラグがセットされている
			# print "(cr) _____________";										# ????
			$self->input_clear();											# 入力バッファを消去
			$self->{'iscr'} = 0;											# 確定フラグをリセット
		}

		# 入力文字と入力文字のデータ長をリセット
		$self->{'inkey'} = '';
		$self->{'inkey_len'} = 0;
		
		# データ取得
		return unless ( $self->_getchar() );		# 文字を取得できない(undef)
		my $inkey = $self->{'inkey'};				# 入力文字
		my $inkey_len = $self->{'inkey_len'};		# 入力文字のデータ長（ＣＰ９３２）

		my $input = $self->{'input'};				# 入力バッファ
		my $input_len = $self->{'input_len'};		# 入力バッファのデータ長（ＣＰ９３２）

		# ● 状態確認用
		# printf "\n** inkey_len=(%s) ", $inkey_len;											# ???? 入力文字のＣＰ９３２データ長
		# printf "\n** inkey=[%s] ", ( $inkey =~ /[\x00-\x1f]/ )								# ???? 入力文字
		# 							? '<'.$self->{'ctrl-code'}->[ ord($inkey) ].'>'
		# 							: encode( 'utf8', decode( 'utf8', $inkey ) );
		# printf "\n** inkey_dump=[ %s ] "
		# 						, join( ' ', map { unpack( "H2", $_ ) } split( //, $inkey ) );	# ???? 入力文字のダンプ
		# printf "\n** input_len=(%d) ", $input_len;											# ???? 入力バッファのＣＰ９３２データ長
		# printf "\n** input=[ %s ] ", encode( 'utf8', $input );								# ???? 入力バッファ


		# ■ データ処理
		#------------------------------------------
		# 入力文字が
		# 	<CR>		入力バッファ確定
		# 	<BS>		入力バッファ後退
		# 	<DEL>		入力バッファ後退
		# 	それ以外	入力バッファ取り込み
		#------------------------------------------

		# 後退処理(BS/DEL)
		if ( $inkey =~ /[\x08|\x7f]/) {															# <BS>, <DEL>
			# print "\n\n** (delete_chr) _____________";											# ????	
			if ( $self->{'input_len'} > 0 ) {														# 入力バッファにデータあり
				my @input = split( //, $input );														# 入力バッファを取得
				my $delchar = pop( @input );															# 入力バッファから１文字除去
					# printf "\n\t** delchar=[%s] ", encode( 'utf8', $delchar );							# ???? 削除文字, 削除文字のＣＰ９３２データ長
				my $delchar_len = cp932len( encode('utf8', $delchar) );									# 削除文字のデータ長（ＣＰ９３２）を取得
					# printf "\n\t** delchar_len=(%d) ", $delchar_len;										# ???? 削除文字, 削除文字のＣＰ９３２データ長
				$self->{'input'} = join( '', @input );													# 入力バッファを更新
					# printf "\n\t** (new) input=[%s] ", encode('utf8', $self->{'input'});					# ???? 入力バッファ（更新後）
				$self->{'input_len'} = ( $input_len - $delchar_len );									# 入力バッファのデータ長から削除した文字のデータ長だけ差し引く
					# printf "\n\t** (new) input_len=(%d) ", $self->{'input_len'};							# ???? 入力バッファのデータ長（更新後）

				if ( $self->{'echo'} == 1 ) {														# エコーバックが有効
					$echo = "\x08 \x08" x $delchar_len;													# 削除文字のデータ長分の後退文字列を返す
				}
			}
			else {																					# 入力バッファにデータなし
				# print "\n\n** (empty) _____________";													# ????
				$echo = "\x07" if ( $self->{'echo'} == 1 );										# エコーバックが有効ならベルを鳴らす(BEL)
			}
			# print "\n\n** (delete_chr-end) _____________";										# ????	
		}

		# 確定処理(CR)
		elsif ( $inkey =~ /\x0d/ ) {															# <CR>
			# print "\n\n** (carridge_return) __________";											# ???? 
			$self->{'iscr'} = 1;																	# 確定フラグをセット
			# printf "\n** input_len=(%d) ", $self->{'input_len'};								# ???? 入力バッファのデータ長（更新後）
			# printf "\n** input=[%s] ", encode('utf8', $self->{'input'});						# ???? 入力バッファ（更新後）
		}

		# 入力文字を入力バッファに追加
		else {																					# <その他の文字>
			my $reject_chr = $self->{'reject_chr'};

			unless ( $inkey =~ /$reject_chr/ ) {													# 入力文字の受け入れ可能
				# printf "\n\n** (accept) ";																# ????	

				if ( $self->{'input_size'} < ( $input_len + $inkey_len ) ) {							# 入力文字のデータ長と取得した文字のデータ長の合計が入力バッファサイズに収まる
					# print "\n\n** (overflow) _____________";												# ????	
																											# ※ データは取りこぼす
					$echo = "\x07" if ( $self->{'echo'} == 1 );										# エコーバックが有効ならベルを鳴らす(BEL)
				}
				else {																					# 入力データが入力バッファに収まらない（オーバーフロー）
					# print "\n\n** (input) _____________";													# ????	
					$self->{'input_len'} = ( $input_len + $inkey_len );										# 入力バッファのデータ長に入力文字のデータ長を加算
					$self->{'input'} .= decode("utf8", $inkey);;											# 入力文字を入力バッファに追加

					# 自動確定(autocr)
					if ( $self->{'autocr'} == 1 ) {
						$self->{'iscr'} = 1 if ( $self->{'input_size'} == $self->{'input_len'} );				# 入力バッファが埋まったら確定フラグをセット
					};


					if ( $self->{'echo'} == 1 ) {														# エコーバックが有効
						# マスクが有効なら入力文字のデータ長分のマスク文字を返す
						# マスクが無効なら入力文字を返す
						$echo = ( $self->{'mask'} == 1 ) ? $self->{'maskchr'} x $inkey_len : $inkey;
					}
					# printf "\n\t** (new) input_len=(%d) ", $self->{'input_len'};							# ???? 入力バッファのデータ長（更新後）
					# printf "\n\t** (new) input=[%s] ", encode('utf8', $self->{'input'});					# ???? 入力バッファ（更新後）
					# printf "\n\t** (new) iscr=(%d) ", $self->{'iscr'};									# ???? 確定フラグ
					# printf "\n\t** (new) echo=[%s] ",$echo;										# ???? 入力バッファのデータ長（更新後）
				}
			}
			else {																					# 入力文字の受け入れ禁止
				# printf "\n\n** (reject) _____________";												# ????
				$echo = "\x07" if ( $self->{'echo'} == 1 );										# エコーバックが有効ならベルを鳴らす(BEL)
			}
		}
	}
	return $echo if (defined($echo) );												# 【 ラインモード終わり 】エコーバックがあれば返す
}

#==================================================
# ■ 受信バッファから１文字取り出す
# chr = _getchar();
# 
# [引数]
# 	なし
# 
# [返り値]
# 	r = 取り出した文字
# 
#==================================================
sub _getchar {
	printf "\n(%s::_getchr [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;

	return if ( $#{ $self->{'recv'} } < 0 );														# 受信バッファにデータがない(undef)

	my $exit 			= 0;					# 終了フラグ
	my @pktdata			= ();					# パケットデータ配列
	my $inkey			= '';					# 入力文字

	# (1) 受信バッファからデータを取り出す
	until ( $exit == 1 ) {																			# 【 取得ここから 】終了フラグが立つまでループ
	    if ( $#pktdata < 0 ) {																			# パケットデータ配列が空
		    $exit = 1 if ( $#{ $self->{'recv'} } < 0 );														# かつ、受信バッファが空(終了)
	        my $packet = shift( @{ $self->{'recv'} } );														# 受信バッファの先頭パケットを取り出す
	        @pktdata = split( //, $packet );																# 取り出したデータをパケットデータ配列に変換
			# my @r = map { unpack( "H2", $_ ) } @pktdata;													# ???? パケットデータ配列の内容
			# printf "\n** [pktdata] = (%d):[ %s ] \n", $#r + 1, join( ', ', @r );							# ???? 
	    }

		# パケットデータ配列からデータを取得
		my $pick = shift( @pktdata );																	# パケットレコードから１バイト取得
		# printf "\n** pick=[%s] ", unpack( "H*", $pick );												# ???? 取得したデータ
		$inkey .= $pick;																				# 取得したデータを入力文字に取り込む

		# ＵＴＦ８文字判定（ざっくり）
		# 文字として識別できるまで取得を繰り返す
		$exit = 1 if ( $inkey =~ /[\xF0-\xF4][\x80-\xBF][\x80-\xBF][\x80-\xBF]/ );						# ４バイト文字
		$exit = 1 if ( $inkey =~ /[\xE0-\xEF][\x80-\xBF][\x80-\xBF]/ );									# ３バイト文字
		$exit = 1 if ( $inkey =~ /[\xC2-\xDF][\x80-\xBF]/ );											# ２バイト文字
		$exit = 1 if ( $inkey =~ /[\x00-\x7f]/ );														# １バイト文字( 制御文字・ASCII )

		# この部分は他の文字コードやプロトコルに合わせ調整できるようにする。
		# 例えば、ＣＰ９３２とか。

	}																								# 【 取得ここまで 】

	# (2) パケットデータ配列を受信バッファに戻す
	if ( $#pktdata >= 0 ) {																			# パケットデータ配列にデータが残っているなら
		unshift( @{ $self->{'recv'} }, join( '', @pktdata ) );											# 受信バッファに戻す
	}

	# (3) 入力文字をセット
	$self->{'inkey'} = $inkey;
	$self->{'inkey_len'} = cp932len( $inkey );														# 入力文字のデータ長（ＣＰ９３２）

	# ● 状態確認
	# my @inkey_ary = split( //, $inkey );															# ???? 入力文字を配列化
	# my $inkey_cp932len = cp932len($inkey);														# ???? 入力文字のデータ長
	# my $inkey_ = ( $inkey =~ /[\x00-\x1f]/ )														# ???? 制御文字を置換
	# 				? '<'.$self->{'ctrl-code'}->[ ord($inkey) ].'>'
	# 				: encode('utf8', decode('utf8', $inkey));
	# printf "\n\n** inkey=\'%s\'\ ; ", $inkey_;													# ???? 入力文字
	# printf "len=";																				# ???? 
	# printf "(cp932:%d),", $inkey_cp932len unless ( $inkey =~ /[\x00-\x1f]/ );						# ???? 入力文字のデータ長（ＣＰ９３２）
	# printf "(utf8:%d) " , ($#inkey_ary + 1);														# ???? 入力文字のデータ長（ＵＴＦ８）
	# printf "; [ %s ] ", join( ' ', map { unpack( "H*", $_ ) } @inkey_ary );						# ???? 入力文字のダンプ

	return $inkey;																					# 取得した文字を返す
}

#==================================================
# ＵＴＦ８文字からＣＰ９３２文字のデータ長を返す
# r = cp932len( utf8str );
# 
# [引数]
# 	utf8str = ＵＴＦ８文字列
# 
# [返り値]
# 	r =
# 		(相互置換に失敗) : -1
# 		(相互置換に成功) : ＣＰ９３２文字のデータ長
# 
# 置換能力はEncodeに依存するが、相互置換できないものもあるので、
# 相互置換を行い、双方のデータが完全一致したら文字長を返す。
# 
# ※この関数は別のパッケージに移動する予定
#==================================================
sub cp932len {
	printf "\n(%s::cp932len [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $utf8_a = shift;															# ＵＴＦ８バイナリ(元データ)
	my $cp932 = encode( "cp932", decode('utf8', $utf8_a) );						# ＣＰ９３２（変換）
	my $utf8_b = encode( "utf8", decode('cp932', $cp932) );						# ＵＴＦ８バイナリ(逆変換)

	# print "\n".Dumper($utf8_a);		# ????
	# print "\n".Dumper($cp932);		# ????
	# print "\n".Dumper($utf8_b);		# ????

	# print "\n[utf8_a]\n";														# ???? ＵＴＦ８(元データ)
	# map { print "$_ " } map { unpack( "H*", $_ ) } split(//, $utf8_a);
	# print "\n[cp932]\n";														# ???? ＣＰ９３２（ＵＴＦ８からの変換）
	# map { print "$_ " } map { unpack( "H*", $_ ) } split(//, $cp932);
	# print "\n[utf8_b]\n";														# ???? ＵＴＦ８（ＣＰ９３２からの逆変換）
	# map { print "$_ " } map { unpack( "H*", $_ ) } split(//, $utf8_b);

	# 相互置換して双方のデータが完全一致するか比較
	my $ok = ( $utf8_a eq $utf8_b ) ? 1 : 0;
	my $cp932len = length($cp932);
	# printf "\nok=(%d) ", $ok;													# ???? 結果
	# printf "\nlen=(%d) ", $cp932len;											# ???? 文字長（ＣＰ９３２）

	return ( $ok == 1 ) ? $cp932len : -1;										# 返り値（完全一致: 文字長（ＣＰ９３２）, 不一致: -1）
}

#==================================================
# ■ 入力バッファをリセット
# r = input_clear();
# 
# [引数]
# 	なし
# 
# [返り値]
# 	なし

#==================================================
sub input_clear {
	printf "\n(%s::input_clear [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;

	# printf "\n** input=(%d):[%s] ", $self->{'input_len'}, encode('cp932', $self->{'input'});			# ????

	$self->{'input'} = '';							# 入力バッファをクリア
	$self->{'input_len'} = 0;						# 入力バッファのデータ長（ＣＰ９３２）をリセット

	# printf "\n** (after) input=(%d):[%s] ", $self->{'input_len'}, encode('cp932', $self->{'input'});	# ????
}

#==================================================
# ■ 受信バッファにデータを取り込む
# store( data );
# 
# [引数]
# 	data : データ
# 
# [返り値]
# 	なし
#==================================================
sub store {
	printf "\n(%s::store [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	my $data = shift;				# 取り込むデータ（バイナリ）

	my @r = map { unpack("H2", $_) } split(//, $data );								# ???? データの内容
	# printf "\n[data] : (%d):[ %s ] ", $#r+1, join(', ', @r);						# ????

	push( @{ $self->{'recv'} }, $data );											# 受信バッファに取り込む

	# print "\n\n[recv]";															# ???? 受信バッファの内容 
	# printf " row=(%d) ", $#{ $self->{'recv'} }+1;									# ???? レコード数
	# for my $i (0..$#{ $self->{'recv'} }) {
	# 	printf "\n(%2d):[ %s ] ", $i, join(', ', map { unpack("H*", $_) } split(//, $self->{'recv'}->[$i]));
	# }
	return;
}

#==================================================
# ■ パラメータ設定
# r = config( 'type' => value )
# 
# [引数]
# 	type  : 設定名
# 	value : 設定値
# 
# [返り値]
# 	(undef) : 設定失敗
# 	1       : 設定成功
#==================================================
sub config {
	printf "\n(%s::config [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	my $type = shift;			# 設定名
	my $value = shift;			# 設定値

	$type = lc($type);
	if ( $type =~ /^(autocr|mask|echo|input_size|input_datasize)$/i ) {			# 設定名チェック（規定の名前）
		if ( $type =~ /^(reject_chr|maskchr)$/i ) {										# 設定値が数値でないのもの
			$self->{$type} = $value;														# (設定値は無保証)
			return 1;																		# 設定成功(1)
		}
		else {																			# 設定値が数値のもの
			if ( $value =~ /\d+/ ) {														# 設定値チェック
				$self->{$type} = $value;
				return 1;																		# 設定成功(1)
			}
		}
	}
	return;																			# 設定失敗(undef)
}

#==================================================
# ■ 入力モードの取得・設定
# type = mode()		(取得)
# mode( type )		(設定)
# 
# [引数]
# 	type:
#		'line'  : ラインモード
#		'block' : ブロックモード
# 
# [返り値]
# 	type:
#		'line'  : ラインモード
#		'block' : ブロックモード
#	(undef) : 設定失敗(設定時)
#==================================================
sub mode {
	printf "\n(%s::mode [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;
	my $type = shift;			# 設定値

	unless ( defined($type) ) {									# 参照(引数なし)
		return $self->{'mode'};
	} 
	else {														# 設定(引数あり)
		$type = lc($type);
		return unless ( $type =~ /^(line|block)$/i );				# 設定値が無効(undef)
		$self->{'mode'} = $type;
		return $type;
	}
}

#==================================================
# ■ 入力バッファを返す（ラインモード有効時）
# data = input_dollar()
# 
# [引数]
#	なし
# 
# [返り値]
#	   data = 文字列
#	 (ヌル) = ラインモードが無効
#==================================================
sub input {
	printf "\n(%s::input [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;

	return if ( $self->{'mode'} ne 'line' );		  # ラインモードでない(undef)
	return $self->{'input'};						# 入力バッファを返す
}

#==================================================
# ■ 直前に入力した文字コードを返す（ラインモード有効時）
# data = inkey_dollar()
# 
# [引数]
#	なし
# 
# [返り値]
#    data = 文字コード
#	 (ヌル) = ラインモードが無効
#==================================================
sub inkey {
	printf "\n(%s::inkey [%s]) ", __PACKAGE__, Debug::_caller(caller) if ($::DEBUG =~ /a/);		# ????
	my $self = shift;

	return if ( $self->{'mode'} ne 'line' );		  # ラインモードでない(undef)
	return $self->{'inkey'};						# 入力文字を返す
}

printf("\n(%s) [ %s ] ", $VERSION, __PACKAGE__ ) if ( $::DEBUG =~ /a/ );

#================================================================
=pod

=encoding utf8

=head1 スクリプト名

Input.pm - BBS.pm サブモジュール 【 入力制御 】

=head1 著者

naoit0 <https://www.naoit0.com/projects/bbs_pm/>

=cut
1;